function TractionDrive1_P
%function TractionDrive1_P

assignin('base','V_DC',400);       %Voltage [Volt]
assignin('base','C_f',0.0002);     %Filter capacitor [As/V]
assignin('base','R_f',2);          %Filter resistor [Ohm]
assignin('base','L_f',0.02);       %Filter inductor [Vs/A]
assignin('base','L_field1',0.015); %Field winding 1 inductance [Vs/A]
assignin('base','R_field1',2);     %Field winding 1 resistance [Ohm]
assignin('base','L_arm1',0.03);    %Armature winding 1 inductance [Vs/A]
assignin('base','R_arm1',1.5);     %Armature winding 1 resistance [Ohm]
assignin('base','L_field2',0.015); %Field winding 2 inductance [Vs/A]
assignin('base','R_field2',2);     %Field winding 2 resistance [Ohm]
assignin('base','L_arm2',0.03);    %Armature winding 2 inductance [Vs/A]
assignin('base','R_arm2',1.5);     %Armature winding 2 resistance [Ohm]
assignin('base','k_mach1',0.1);    %1. torque coefficient: if input i_f1
assignin('base','k_mach2',0.1);    %2. torque coefficient: if input i_f2 
assignin('base','mbog1',2);        %First bogie ratio [m^-1]
assignin('base','mbog2',2);        %Second bogie ratio [m^-1]
assignin('base','M',10000);        %Mass of the bogie [kg]
assignin('base','alpha',0);        %Slope angle [rad]
assignin('base','a_res',1);        %Resistive force: linear coefficient
assignin('base','b_res',2);        %Resistive force: quadratic coefficient
assignin('base','g',9.81);         %gravity constant [m/s^2]

assignin('base','i_filter_0',0);   %i_filter initial value [A]
assignin('base','u_filter_0',0);   %u_filter initial value [V]
assignin('base','i_field1_0',0);   %i_field1 initial value [A]
assignin('base','i_field2_0',0);   %i_field2 initial value [A]
assignin('base','i_arm_0',0);      %i_arm initial value [A]
evalin('base','I_M_0=[i_field1_0 i_arm_0 i_field2_0];'); %i_vector IC [A]
assignin('base','v_sub_0',30/3.6); %Subway initial velocity [m/s]=[30km/h]

assignin('base','m_chop',[0.3 0.9 0.3]);       %Switching functions
assignin('base','m_chop_phase',[0 0.33 0.66]); %Phase of the choppers
assignin('base','Ts',0.00002);                 %Switching period

                           %Matrix / vector definition in MATLAB workspace
evalin('base','L_M=diag([L_field1 L_arm1+L_arm2 L_field2]);');%Ind. matrix
evalin('base','R_M=diag([R_field1 R_arm1+R_arm2 R_field2]);');%Res. matrix
evalin('base','M_b=[mbog1 mbog2];');               %Bogie ratio matrix
evalin('base','M_k=[k_mach1/L_field1 0 0; 0 k_mach2/L_field2 0];'); 
                                                   %valid for input: Psi

txt='"TractionSystem 1"';
disp(['Parameter for Bond Graph example ' txt ' loaded!']);
