function [h,bh]=BGV2_TSI( h, R )
% h:  Handle
% R:  Richtung der Suche - (1=vorwrts, 0=rckwrts)
% bh: Pfad des Ziel-Subsystems
%     Wird nur fr Ports erzeugt und enthlt am Ende den vollstndigen Pfad
%     mit dem Blocknamen des Subsystems in dem sich der Port befindet. 

% TRACE_SIGNAL( handle, direction )
%   goes to Simulink object specified by HANDLE argument (block, port or line)
%   and returns array of handles when following according to DIRECTION:
%                                                      (1=forward, 0=backward)
%   Example
%     x={gcb};while length(x)==1; x=trace_signal(x,0); end % follow signal
%     until signal branch or sink

bh=0; % Anfangswert
      if isempty(h)
        return
      end
      if iscell(h) 
        h=h{1};
      end
      if R
          switch_expr=[get_param(h,'Type') '->'];
      else
          switch_expr=['<-' get_param(h,'Type')];
      end 
      
      switch switch_expr    
      case 'line->'
             h=get_param( h, 'DstPortHandle');      
      case '<-line'             
             h=get_param( h, 'SrcPortHandle');
             
      case 'port->'
         bh=get_param( h, 'Parent'); % >>>Ausgabe fr BG v.2.0<<<

         if strcmp('outport', get_param(h,'PortType'));
            % we leaved subsystem via outport
             h=find_system( get_param(get_param(h,'Parent'),'Parent'), 'SearchDepth',1,'LookUnderMasks','all','FindAll', 'on', ...
                                'type', 'line','SrcPortHandle', h); 
         else
            % we enter block  via inport
            switch get_param(bh,'BlockType')
               case 'SubSystem'
               % enter subsystem = find corresponding Inport block
                phs=get_param(bh,'PortHandles');
                portNr=find(phs.Inport==h);
                h=find_system( bh, 'SearchDepth',1,'LookUnderMasks','all', 'BlockType', 'Inport', ...
                                  'Port', num2str(portNr));
             otherwise
                h={bh};
             end %switch
         
         end

         

          
      case '<-port'
         bh=get_param(h,'Parent'); % >>>Ausgabe fr BG v.2.0<<<
         if strcmp('outport', get_param(h,'PortType'));
             % we enter subsystem via outport
             
             switch get_param(bh,'BlockType')
                  case 'SubSystem' 
                      phs=get_param( get_param(h,'Parent'), 'PortHandles');
                      portNr=find(phs.Outport==h);
                      h=find_system(bh, 'SearchDepth',1,'LookUnderMasks','all', 'BlockType', 'Outport', ...
                                  'Port', num2str(portNr));
                  otherwise
                      h={bh};
              end        
         else 
                h=find_system( get_param(get_param( h,'Parent'),'Parent'), 'SearchDepth',1,'LookUnderMasks','all','FindAll', 'on', ...
                                'type', 'line','DstPortHandle', h); 
% 
%              if strcmp('Outport', get_param(bh,'BlockType'))
%                 % we entered subsystem via outport 
%                 h=find_system( get_param(get_param( h,'Parent'),'Parent'), 'SearchDepth',1,'FindAll', 'on', ...
%                                 'type', 'line','DstPortHandle', h); 
% 
%              else 
%                 % we leaved subsystem via inport 
%                 h=find_system( get_param(get_param( h,'Parent'),'Parent'), 'SearchDepth',1,'FindAll', 'on', ...
%                                 'type', 'line','DstPortHandle', h); 
%              end
         end
        
      case 'block->'
            switch get_param(h,'BlockType')
                case 'Outport'
                    % find corresponding subsystem port
                    phs=get_param( get_param(h,'Parent'), 'PortHandles'); 
                    portNr=str2num(get_param( h,'Port'));    
                    h=phs.Outport(portNr); % single

                otherwise
                    phs=get_param(h, 'PortHandles'); 
                    h=phs.Outport;  % all 
            end
            
            
      case '<-block'
            switch get_param(h,'BlockType')
                case 'Inport'
                    
                    % find corresponding subsystem port
                    phs=get_param(get_param(h,'Parent'), 'PortHandles'); 
                    portNr=str2num(get_param( h,'Port')); 
                    h=phs.Inport(portNr); % single

                otherwise
                    phs=get_param(h, 'PortHandles');
                    h=phs.Inport;  % all 
            end
            
            
      end % switch

% always return cellarray:
if ~iscell(h)
    r={};
	for i=1:length(h); r{i}=h(i); end
    h=r;
end;



% quick+dirty logging: 
%for i=1:length(h)

%        if isnumeric( h{i}) && h{i}<0; type='?' ; else type=  get_param(h{i},'type'); end;
%        fprintf('%%    %2d. %5s%5s : %s\n', i, type ,' ', h{i});

%end


