function BGV2_KNU
%Umschaltung eines KN-Blocks auf Verteiler-Funktion, d.h. alternative
%Eingangsvariable fr Leistung
%Voraussetzung: ZU==1 und AB>1 mu bereits bei Eingabe geprft werden.
%Neu: MenVariable BI={off/on}: Wahlweise erster Eingangsport mit
%                               bidirektionalem negativen Vorzeichen

FT00='KN Element - BGV2_KNU';
FT01=['Parameter access denied!' 10 10 'Pre-Condition for alternative input is: '];
FT02='"Fed Power Ports(FPP number)"==1!';
FT03='"Consumed Power Ports (CPP number)">=2!';
FT04='Cancellation of "Alternative power variable input": ';
FT05='false KN-Element kode detected!';
FT06='Blocklogic: Fatal error - please delete this block!';

WT01='Invalid mask parameters: Auto Reset for Bidirectional minus sign!';
WT02=[10 10 'Selection is valid only for: alternative power variable input!'];

h=gcb;
h2=gcs;
bln=get_param(h,'Name');
blks=find_system(h,'FollowLinks','on','LookUnderMasks','all','BlockType','Gain');
[AnzBlo,xx]=size(blks); %0=Normalfkt,1=nur_umgewandelt,2=umgewandelt+BIDIRMI
FieldNames=fieldnames(get_param(h,'Dialogparameters')); % Alte NodeBlockVersion ?
if   sum(strcmp(FieldNames(:),'BI')), BI_str=get_param(h,'BI'); %Schon BI existent?
else                                  BI_str='off'; end %indirekt, sowieso String
if strcmp(get_param(h,'UM'),'on'),        %vorwrts: Normalfunktion --> Verteiler
   if AnzBlo>0,                           %Umwandlung schon ausgefhrt?
      if AnzBlo==1&&strcmp(get_param(blks(1),'Name'),'Y'),
         errordlg([h 10 10 FT06],FT00);   %Korrektur nicht mglich
         return; end
      Kode=get_param([h '/K'],'Value');                        %Welcher KnotenTyp?
      if   strcmp(Kode,'61')||strcmp(Kode,'63'), N2='F1';      %1-Knoten
      else                                       N2='E1'; end; %0-Knoten
      BIDIRMI(h,h2,bln,N2,AnzBlo,BI_str); %AnzBlo ist 1 oder 2
   else                                   %AnzBlo==0 erkannt
      Sum=get_param([h '/S'],'Inputs');
      Plus=findstr(Sum,'+'); Minus=findstr(Sum,'-');
      if length(Plus)~=1,
                    errordlg([h 10 10 FT01 FT02],FT00); 
                    set_param(h,'UM','off'); return; end
      if length(Minus)<2,
                    errordlg([h 10 10 FT01 FT03],FT00); 
                    set_param(h,'UM','off'); return; end
      if   strcmp(get_param([h '/K'],'Value'),'61') %Knotentyp?
           N0='E0'; N1='E1'; N2='F1';        % 1-Knoten
      else N0='F0'; N1='F1'; N2='E1'; end    % 0-Knoten
      delete_line( [h2 '/' bln] , 'K0/1', 'K1/1' );%KnotenTyp-unabhngige Umwandlungn
      add_block( 'built-in/Gain', [h '/X' ], 'Gain', '-1' );
      add_line( [h2 '/' bln] , 'S/1', 'X/1' );
      add_line( [h2 '/' bln] , 'X/1', 'K1/1' );%Vom KnotenTyp abhngige Umwandlungen:
      replace_block(h,'FollowLinks','on','Name',N0,'built-in/Terminator','noprompt');
      replace_block(h,'FollowLinks','on','Name',N1,'built-in/Ground','noprompt');
      replace_block(h,'FollowLinks','on','Name','K0','built-in/Inport','noprompt');
      set_param([h '/K0'],'Name',N2);
      BIDIRMI(h,h2,bln,N2,1,BI_str);         %1==schon_umgewandelt
   end
else                                         %rckwrts: Verteiler --> Normalfunktion
      if AnzBlo==0, return; end              %Umwandlung schon ausgefhrt
      if AnzBlo==1&&strcmp(get_param(blks(1),'Name'),'Y'),
         errordlg([h 10 10 FT06],FT00);      %Korrektur nicht mglich
         return; end
      if strcmp(BI_str,'on')                 %BI ist schon off eingestellt?
         warndlg([h 10 10 WT01 WT02],FT00);  %falsche Meneinstellung!
         set_param(h,'BI','off'); end        %BI wird off gesetzt!
      Kode=get_param([h '/K'],'Value');      %KnotenTyp?
      if     strcmp(Kode,'63'), N0='E0'; N1='E1'; N2='F1'; % 1-Knoten
      elseif strcmp(Kode,'62'), N0='F0'; N1='F1'; N2='E1'; % 0-Knoten
      else   errordlg([h 10 10 FT04 FT05],FT00); return; end
      if AnzBlo==2, BIDIRMI(h,h2,bln,N2,AnzBlo,BI_str); end %nur aufrufen wenn ntig
      set_param([h '/' N2],'Name','K0');     %Vom KnotenTyp abhngige Umwandlungen:
      replace_block(h,'FollowLinks','on','Name','K0','built-in/From','noprompt');
      set_param([h '/K0'],'GotoTag','XXX');
      replace_block(h,'FollowLinks','on','Name',N1,'built-in/Inport','noprompt');
      replace_block(h,'FollowLinks','on','Name',N0,'built-in/Outport','noprompt');
      set_param([h '/' N0],'Port','1');      %Vom KnotenTyp unabhngige Umwandlungen:
      delete_line( [h2 '/' bln] , 'X/1', 'K1/1' );
      delete_line( [h2 '/' bln] , 'S/1', 'X/1' );
      delete_block([h '/X' ]);
      add_line( [h2 '/' bln] , 'K0/1', 'K1/1' );
end

function BIDIRMI(h,h2,bln,N2,AnzBlo,BI_str)
%wahlweiser Aufruf von BIDIRMI_EIN oder BIDIRMI_AUS
if   strcmp(BI_str,'on')                           %BI=on: off/on
     if AnzBlo ==1, BIDIRMI_EIN(h,h2,bln,N2); end; %schon on
else                                               %BI=off: on/off
     if AnzBlo ==2, BIDIRMI_AUS(h,h2,bln,N2); end; %schon off
end

function BIDIRMI_EIN(h,h2,bln,N2)
%Einschalten von birirektionalem negativen Vorzeichen fr ersten Inport
%alle Umwandlungen unabhngig vom Knotentyp!
delete_line( [h2 '/' bln] , [N2 '/1'], [N2(1) '2/1'] );
add_block( 'built-in/Gain', [h '/Y' ], 'Gain', '-1' );
add_line( [h2 '/' bln] , [N2 '/1'], 'Y/1' );
add_line( [h2 '/' bln] , 'Y/1', [N2(1) '2/1']  ); %Minus F2(1-N)/E2(0-N)-Ausgang
Sum=get_param([h '/S'],'Inputs');
Sum(2)='+';
set_param([h '/S'],'Inputs',Sum); %Minus K2=E2(1-N)/K2=F2(0-N)-Konnektor

function BIDIRMI_AUS(h,h2,bln,N2)
%Einschalten von birirektionalem negativen Vorzeichen fr ersten Inport
%alle Umwandlungen unabhngig vom Knotentyp!
delete_line( [h2 '/' bln] , 'Y/1', [N2(1) '2/1']  );
delete_line( [h2 '/' bln] , [N2 '/1'], 'Y/1' );
delete_block( [h '/Y' ] );
add_line( [h2 '/' bln] , [N2 '/1'], [N2(1) '2/1'] ); %Plus F2(1-N)/E2(0-N)-Ausgang
Sum=get_param([h '/S'],'Inputs');
Sum(2)='-';
set_param([h '/S'],'Inputs',Sum); %Plus K2=E2(1-N)/K2=F2(0-N)-Konnektor
